@@ -100,8 +100,12 @@ my $configure_php=sub {
    my $recaptcha_publish_key=$_[6]||'';
    my $recpatcha_secret_key=$_[7]||'';
    my ($stdout,$stderr)=('','');
-   my $handle=$localhost;my $connect_error='';
-   my $sudo=($^O eq 'cygwin')?'':'sudo ';
+   my $handle=connect_shell();my $connect_error='';
+   my $build_php=0;
+   my $sudo=($^O eq 'cygwin')?'':
+         'sudo env "LD_LIBRARY_PATH='.
+         '/usr/local/lib64:$LD_LIBRARY_PATH" '.
+         '"PATH=/usr/local/mysql/scripts:$PATH" ';
    my $prompt=$handle->prompt();
    ($stdout,$stderr)=$handle->cmd($sudo.'rm -rvf /var/cache/yum',
       '__display__');
@@ -142,14 +146,16 @@ my $configure_php=sub {
       ($stdout,$stderr)=$handle->cmd($sudo.
          "yum -y groupinstall 'Development tools'",'__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'yum -y install openssl-devel icu cyrus-sasl '.
+         'yum -y install icu cyrus-sasl openssl-devel'.
          ' cyrus-sasl-devel libtool-ltdl-devel libjpeg-turbo-devel'.
          ' freetype-devel libpng-devel java-1.7.0-openjdk-devel'.
          ' unixODBC unixODBC-devel libtool-ltdl libtool-ltdl-devel'.
          ' ncurses-devel xmlto autoconf libmcrypt libmcrypt-devel'.
          ' libcurl libcurl-devel libicu libicu-devel re2c'.
-         ' libpng-devel.x86_64 freetype-devel.x86_64 cmake'.
-         ' oniguruma oniguruma-devel tcl tcl-devel git-all',
+         ' libpng-devel.x86_64 freetype-devel.x86_64 expat-devel'.
+         ' oniguruma oniguruma-devel tcl tcl-devel git-all'.
+         ' lzip libffi-devel libc-client-devel texinfo cmake'.
+         ' systemd-devel bind-utils mailx',
          '__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
          'yum -y update','__display__');
@@ -157,7 +163,7 @@ my $configure_php=sub {
          'yum -y install yum-utils','__display__');
    }
 #cleanup;
-   ($stdout,$stderr)=$handle->cmd("wget -qO- https://icanhazip.com");
+   ($stdout,$stderr)=$handle->cmd("wget -qO- http://icanhazip.com");
    $public_ip=$stdout if $stdout=~/^\d+\.\d+\.\d+\.\d+\s*/s;
    unless ($public_ip) {
       require Sys::Hostname;
@@ -181,30 +187,90 @@ my $configure_php=sub {
       '__display__');
    ($stdout,$stderr)=$handle->cwd('/opt/source');
    ($stdout,$stderr)=$handle->cmd($sudo.
-      'git clone git://sourceware.org/git/bzip2.git',
-      '__display__');
-   ($stdout,$stderr)=$handle->cwd('bzip2');
+      'wget -qO- https://www.sourceware.org/bzip2/');
+   $stdout=~s/^.*?stable version is bzip2 ([\d\.]*\d)\..*$/$1/s;
    ($stdout,$stderr)=$handle->cmd($sudo.
-      'make -f Makefile-libbz2_so','__display__');
-   ($stdout,$stderr)=$handle->cmd($sudo.
-      'cp -v libbz2.so* /usr/local/lib','__display__');
-   ($stdout,$stderr)=$handle->cmd($sudo.
-      'make','__display__');
-   ($stdout,$stderr)=$handle->cmd($sudo.
-      'make install','__display__');
+      "ls -1 /usr/local/lib | grep libbz2.so.$stdout");
+   unless ($stdout) {
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'ls -1 | grep bzip2');
+      if ($stdout=~/^\s*bzip2\s*$/s) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+             'rm -rvf bzip2-old','__display__');
+         ($stdout,$stderr)=$handle->cmd($sudo.
+             'mv -v bzip2 bzip-old','__display__');
+      }
+      my $done=0;my $gittry=0;
+      while ($done==0) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+            'git clone git://sourceware.org/git/bzip2.git',
+            '__display__');
+         if (++$gittry>5) {
+            print "\n\n   FATAL ERROR: $stderr\n\n";
+            cleanup();
+         }
+         my $gittest='Connection reset by peer|'.
+                     'Could not read from remote repository';
+         $done=1 if $stderr!~/$gittest/s;
+         last if $done;
+         sleep 30;
+      }
+      ($stdout,$stderr)=$handle->cwd('bzip2');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'make -f Makefile-libbz2_so','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'cp -v libbz2.so* /usr/local/lib','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'make','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'make install','__display__');
+      $build_php=1;
+   } else {
+      print "bzip2 is up to date.\n";
+   }
    ($stdout,$stderr)=$handle->cwd('/opt/source');
    ($stdout,$stderr)=$handle->cmd($sudo.
-      'git clone https://gitlab.gnome.org/GNOME/libxml2.git',
-      '__display__');
-   ($stdout,$stderr)=$handle->cwd('libxml2');
+      'wget -qO- http://xmlsoft.org/news.html');
+   $stdout=~s/^.*?public releases.*?v(.*?):.*$/$1/s;
    ($stdout,$stderr)=$handle->cmd($sudo.
-      './autogen.sh','__display__');
-   ($stdout,$stderr)=$handle->cmd($sudo.
-      'make','__display__');
-   ($stdout,$stderr)=$handle->cmd($sudo.
-      'make install','__display__');
-   ($stdout,$stderr)=$handle->cmd($sudo.
-      'cp -v libxml-2.0.pc /usr/lib64/pkgconfig','__display__');
+      "ls -1 /usr/local/lib | grep libxml2.so.$stdout");
+   unless ($stdout) {
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'ls -1 | grep libxml2');
+      if ($stdout=~/^\s*libxml2\s*$/s) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+             'rm -rvf libxml2-old','__display__');
+         ($stdout,$stderr)=$handle->cmd($sudo.
+             'mv -v libxml2 libxml2-old','__display__');
+      }
+      my $done=0;my $gittry=0;
+      while ($done==0) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+            'git https://gitlab.gnome.org/GNOME/libxml2.git',
+            '__display__');
+         if (++$gittry>5) {
+            print "\n\n   FATAL ERROR: $stderr\n\n";
+            cleanup();
+         }
+         my $gittest='Connection reset by peer|'.
+                     'Could not read from remote repository';
+         $done=1 if $stderr!~/$gittest/s;
+         last if $done;
+         sleep 30;
+      }
+      ($stdout,$stderr)=$handle->cwd('libxml2');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         './autogen.sh','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'make','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'make install','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'cp -v libxml-2.0.pc /usr/lib64/pkgconfig','__display__');
+      $build_php=1;
+   } else {
+      print "libxml2 is up to date.\n";
+   }
    ($stdout,$stderr)=$handle->cwd('/opt/source');
    ($stdout,$stderr)=$handle->cmd($sudo.
       'wget --random-wait --progress=dot '.
@@ -221,6 +287,7 @@ my $configure_php=sub {
       'make install','__display__');
    ($stdout,$stderr)=$handle->cmd($sudo.
       'cp -v sqlite3.pc /usr/lib64/pkgconfig','__display__');
+
    ($stdout,$stderr)=$handle->cwd('/opt/source');
    ($stdout,$stderr)=$handle->cmd($sudo.
       'wget --random-wait --progress=dot '.
@@ -238,6 +305,70 @@ my $configure_php=sub {
    ($stdout,$stderr)=$handle->cmd($sudo.'make','__display__');
    ($stdout,$stderr)=$handle->cmd($sudo.'make install','__display__');
    ($stdout,$stderr)=$handle->cwd('/opt/source');
+   ($stdout,$stderr)=$handle->cmd($sudo.
+      'wget -qO- https://en.wikipedia.org/wiki/OpenSSL');
+   $stdout=~s/^.*?Stable release.*?-data["][>](.*?) *[(].*$/$1/s;
+   my $osslv=$stdout;
+   ($stdout,$stderr)=$handle->cmd($sudo.
+      'strings /usr/local/lib64/libssl.so | grep OpenSSL');
+   unless ($stdout=~/$osslv/s) {
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'ls -1 | grep openssl');
+      if ($stdout=~/^\s*openssl\s*$/s) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+             'rm -rvf openssl-old','__display__');
+         ($stdout,$stderr)=$handle->cmd($sudo.
+             'mv -v openssl openssl-old','__display__');
+      }
+      my $done=0;my $gittry=0;
+      while ($done==0) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+            'git clone --recursive https://github.com/openssl/openssl.git',
+            '__display__');
+         if (++$gittry>5) {
+            print "\n\n   FATAL ERROR: $stderr\n\n";
+            cleanup();
+         }
+         my $gittest='Connection reset by peer|'.
+                     'Could not read from remote repository';
+         $done=1 if $stderr!~/$gittest/s;
+         last if $done;
+         sleep 30;
+      }
+      ($stdout,$stderr)=$handle->cwd('openssl');
+      # https://www.thegeekstuff.com/2015/02/rpm-build-package-example/
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'wget --random-wait --progress=dot '.
+         'https://git.sailfishos.org/mer-core/'.
+         'openssl/raw/master/rpm/openssl.spec',
+         '__display__');
+      $osslv=~s/\./_/g;
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         "git checkout OpenSSL_$osslv",'__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         './config LDFLAGS="-Wl,-rpath /usr/local/lib -Wl,'.
+         '-rpath /usr/local/lib64"','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'make install','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'cp -Pv /etc/ssl/certs/* /usr/local/ssl/certs',
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'cp -v *.pc /usr/local/lib/pkgconfig',
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'ldconfig -v','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'ln -s /usr/local/lib64/libssl.so.1.1 '.
+         '/usr/lib64/libssl.so.1.1');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'ln -s /usr/local/lib64/libcrypto.so.1.1 '.
+         '/usr/lib64/libcrypto.so.1.1');
+      $build_php=1;
+   } else {
+      print "libssl is up to date.\n";
+   }
+   ($stdout,$stderr)=$handle->cwd('/opt/source');
    ($stdout,$stderr)=$handle->cmd('wget --version');
    $stdout=~s/^.*?\d[.](\d+).*$/$1/s;
    if ($stdout<18 && !(-e '/usr/local/bin/wget')) {
@@ -250,17 +381,27 @@ my $configure_php=sub {
       ($stdout,$stderr)=$handle->cwd("wget-*");
       ($stdout,$stderr)=$handle->cmd($sudo.
          './configure --prefix=/usr/local '.
-         '--sysconfdir=/etc --with-ssl=openssl',
+         '--sysconfdir=/etc --with-ssl=openssl '.
+         '--with-libssl-prefix=/usr/local ',
          '__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
          'make','__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
          'make install','__display__');
+      $ad='ca-certificate = /usr/local/ssl/certs/ca-bundle.crt';
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         "sed -i \'/remoteencoding/a$ad\' /etc/wgetrc");
    }
-   ($stdout,$stderr)=$handle->cwd("/opt/source");
-   # https://shaunfreeman.name/compiling-php-7-on-centos/
-   # https://www.vultr.com/docs/how-to-install-php-7-x-on-centos-7
-   if (-1==index `php -v`,'PHP') {
+
+   ($stdout,$stderr)=$handle->cwd('/opt/source');
+   #
+   #  Set PHP 7 or 8 here
+   #
+   #  roundcube does not work with php 8 as of 7/8/2021
+   my $vn=7;
+   ($stdout,$stderr)=$handle->cmd($sudo.
+      'strings /usr/local/lib/libmcrypt.so | grep libmcrypt-2.5.8');
+   unless ($stdout) {
       ($stdout,$stderr)=$handle->cmd($sudo.
          'wget --random-wait --progress=dot -O libmcrypt-2.5.8.tar.gz '.
          'https://sourceforge.net/projects/mcrypt/files/Libmcrypt/2.5.8/'.
@@ -272,37 +413,86 @@ my $configure_php=sub {
          './configure','__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
          'make install','__display__');
-      ($stdout,$stderr)=$handle->cwd('/opt/source');
-      ($stdout,$stderr)=$handle->cmd($sudo.
-         'cmake --version','__display__');
-      $stdout=~s/^.*?\s(\d+\.\d+).*$/$1/;
-      if (!(-e '/usr/local/bin/cmake') && $stdout<3.02) {
+      $build_php=1;
+   } else {
+      print "libmcrypt is up to date\n";
+   }
+   ($stdout,$stderr)=$handle->cwd('/opt/source');
+   ($stdout,$stderr)=$handle->cmd($sudo.
+      'cmake --version','__display__');
+   $stdout=~s/^.*?\s(\d+\.\d+).*$/$1/;
+   if (!(-e '/usr/local/bin/cmake') && $stdout<3.02) {
+      my $done=0;my $gittry=0;
+      while ($done==0) {
          ($stdout,$stderr)=$handle->cmd($sudo.
             'git clone https://github.com/Kitware/CMake.git',
             '__display__');
-         ($stdout,$stderr)=$handle->cwd('CMake');
-         ($stdout,$stderr)=$handle->cmd($sudo.
-            './configure','__display__');
-         ($stdout,$stderr)=$handle->cmd($sudo.
-            'make','__display__');
-         ($stdout,$stderr)=$handle->cmd($sudo.
-            'make install','__display__');
-         ($stdout,$stderr)=$handle->cwd('/opt/source');
+         if (++$gittry>5) {
+            print "\n\n   FATAL ERROR: $stderr\n\n";
+            cleanup();
+         }
+         my $gittest='Connection reset by peer|'.
+                     'Could not read from remote repository';
+         $done=1 if $stderr!~/$gittest/s;
+         last if $done;
+         sleep 30;
       }
+      ($stdout,$stderr)=$handle->cwd('CMake');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'git clone https://github.com/nih-at/libzip.git',
+         './bootstrap --system-curl -- '.
+         '-DCMAKE_INSTALL_RPATH="/usr/local/lib64"',
          '__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'make','3600','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'make install','__display__');
+      $build_php=1;
+   } else {
+      print "cmake is up to date.\n";  
+   }
+   ($stdout,$stderr)=$handle->cwd('/opt/source');
+   ($stdout,$stderr)=$handle->cmd($sudo.
+      'wget -qO- https://libzip.org/');
+   $stdout=~s/^.*?Current version is (.*?)[<].*$/$1/s;
+   $stdout='1.6.1';
+   ($stdout,$stderr)=$handle->cmd($sudo. 
+      "strings /usr/local/lib64/libzip.so | grep $stdout");
+   unless ($stdout) {
+      my $done=0;my $gittry=0;
+      while ($done==0) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+            'git clone https://github.com/nih-at/libzip.git',
+            '__display__');
+         if (++$gittry>5) {
+            print "\n\n   FATAL ERROR: $stderr\n\n";
+            cleanup();
+         }
+         my $gittest='Connection reset by peer|'.
+                     'Could not read from remote repository';
+         $done=1 if $stderr!~/$gittest/s;
+         last if $done;
+         sleep 30;
+      }
       ($stdout,$stderr)=$handle->cwd('libzip');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'git tag -l','__display__');
-      $stdout=~s/^.*\n(.*)$/$1/s;
+         'git -P tag -l','__display__');
+      $stdout=~s/^.*\n(rel-\d-\d-\d).*$/$1/s;
+$stdout='rel-1-6-1';
+#$stdout='v1.8.0';
       ($stdout,$stderr)=$handle->cmd($sudo.
          "git checkout $stdout",'__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
          'mkdir -vp build','__display__');
       ($stdout,$stderr)=$handle->cwd('build');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         '/usr/local/bin/cmake ..','__display__');
+         '/usr/local/bin/cmake .. '.
+         '-DCMAKE_SHARED_LINKER_FLAGS="-L/usr/local/lib64" '.
+         '-DCMAKE_INSTALL_RPATH="/usr/local/lib64" '.
+         '-DOPENSSL_INCLUDE_DIR=/usr/local/include/openssl '.
+         '-DOPENSSL_SSL_LIBRARY=/usr/local/lib64/libssl.so '.
+         '-DOPENSSL_CRYPTO_LIBRARY='.
+         '/usr/local/lib64/libcrypto.so',
+         '__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
          'make','__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
@@ -310,10 +500,42 @@ my $configure_php=sub {
       ($stdout,$stderr)=$handle->cmd($sudo.
          'cp -v libzip.pc /usr/lib64/pkgconfig',
          '__display__');
-      ($stdout,$stderr)=$handle->cwd('/opt/source');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'git clone https://github.com/jedisct1/libsodium',
-         '__display__');
+         'ldconfig -v','__display__');
+      $build_php=1;
+   } else {
+      print "libzip is up to date\n";
+   }
+   ($stdout,$stderr)=$handle->cwd('/opt/source');
+   ($stdout,$stderr)=$handle->cmd($sudo.
+      'wget -qO- https://doc.libsodium.org/#downloading-libsodium');
+   $stdout=~s/^.*?libsodium (.*?)-stable.*$/$1/s;
+   ($stdout,$stderr)=$handle->cmd($sudo.
+      "strings /usr/local/lib/libsodium.so | grep $stdout");
+   unless ($stdout) {
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'ls -1 | grep libsodium');
+      if ($stdout=~/^\s*libsodium\s*$/s) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+             'rm -rvf libsodium-old','__display__');
+         ($stdout,$stderr)=$handle->cmd($sudo.
+             'mv -v libsodium libsodium-old','__display__');
+      }
+      my $done=0;my $gittry=0;
+      while ($done==0) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+            'git clone https://github.com/jedisct1/libsodium 2>&1',
+            '__display__');
+         if (++$gittry>5) {
+            print "\n\n   FATAL ERROR: $stderr\n\n";
+            cleanup();
+         }
+         my $gittest='Connection reset by peer|'.
+                     'Could not read from remote repository';
+         $done=1 if $stderr!~/$gittest/s && $stdout!~/$gittest/s;
+         last if $done;
+         sleep 30;
+      }
       ($stdout,$stderr)=$handle->cwd('libsodium');
       ($stdout,$stderr)=$handle->cmd($sudo.
          'git checkout -b remotes/origin/stable',
@@ -327,34 +549,74 @@ my $configure_php=sub {
       ($stdout,$stderr)=$handle->cmd($sudo.
          'cp -v libsodium.pc /usr/lib64/pkgconfig',
          '__display__');
-      ($stdout,$stderr)=$handle->cwd('/opt/source');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'git clone https://github.com/php/php-src.git',
-         '__display__');
+         'ldconfig -v','__display__');
+      $build_php=1;
+   } else {
+      print "libsodium is up to date.\n";
+   }
+   ($stdout,$stderr)=$handle->cwd('/opt/source');
+   ($stdout,$stderr)=$handle->cmd($sudo.
+      'wget -qO- https://www.php.net/releases/index.php');
+   $stdout=~s/^.*?php-($vn.*?)\.tar\.gz.*$/$1/s;
+   my $phpv=$stdout;
+   ($stdout,$stderr)=$handle->cmd($sudo.
+      'ls -1 | grep php-src$');
+   my $php_build=0;
+   if ($stdout=~/^\s*php-src\s*$/s) {
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'head -n5 php-src/NEWS');
+      $stdout=~s/^.*, PHP (.*?)\n.*$/$1/s;
+      unless ($phpv eq $stdout) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+            'rm -rvf php-src-old','__display__');
+         ($stdout,$stderr)=$handle->cmd($sudo.
+            'mv -v php-src php-src-old','__display__');
+         $php_build=1;
+      }
+   } else { $php_build=1 }
+   if ($php_build || $build_php) {
+      my $done=0;my $gittry=0;
+      while ($done==0) {
+         ($stdout,$stderr)=$handle->cmd($sudo.
+            'git clone https://github.com/php/php-src.git',
+            '__display__');
+         if (++$gittry>5) {
+            print "\n\n   FATAL ERROR: $stderr\n\n";
+            cleanup();
+         }
+         my $gittest='Connection reset by peer|'.
+                     'Could not read from remote repository';
+         $done=1 if $stderr!~/$gittest/s;
+         last if $done;
+         sleep 30;
+      }
       ($stdout,$stderr)=$handle->cwd('php-src');
-      # https://clipbucket.com/cb-install-requirements/
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'git branch -a');
-      my @phpbranch=map { $_ =~ s/^.*(PHP.*)/$1/ ? $_ : () }
-         split /\n/, $stdout;
-      ($stdout,$stderr)=$handle->cmd($sudo.
-         "git checkout $phpbranch[$#phpbranch]",'__display__');
+         "git checkout php-$phpv",'__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
          './buildconf --force','__display__');
+      my $pear=($vn eq 8)?'--with-pear ':'';
       ($stdout,$stderr)=$handle->cmd($sudo.
-         './configure --prefix=/usr/local/php7 '.
-         '--with-config-file-path=/usr/local/php7/etc '.
-         '--with-config-file-scan-dir=/usr/local/php7/etc/conf.d '.
+         './configure --prefix=/usr/local/php'.$vn.' '.
+         '--with-config-file-path=/usr/local/php'.$vn.'/etc '.
+         '--with-config-file-scan-dir=/usr/local/php'.$vn.'/etc/conf.d '.
          '--enable-bcmath '.
          '--with-bz2 '.
          '--with-curl '.
+         $pear.
          '--enable-filter '.
          '--enable-fpm '.
+         '--with-fpm-systemd '.
          '--enable-gd '.
          '--with-freetype '.
+         '--with-imap '.
+         '--with-imap-ssl '.
          '--with-jpeg '.
          '--enable-intl '.
+         '--enable-exif '.
          '--enable-mbstring '.
+         '--with-gmp '.
          '--with-sodium '.
          '--enable-mysqlnd '.
          '--with-mysql-sock=/var/lib/mysql/mysql.sock '.
@@ -370,112 +632,106 @@ my $configure_php=sub {
          '--enable-xmlreader '.
          '--enable-xmlwriter '.
          '--with-zip '.
-         '--with-zlib','__display__');
-      ($stdout,$stderr)=$handle->cmd($sudo.'make -j2','__display__');
-cleanup;
+         '--with-zlib '.
+         '--with-libdir=lib64 '.
+         '--with-kerberos','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.'make -j2',300,'__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.'make install','__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'ln -s /usr/local/php7/bin/php /usr/local/bin/php');
+         'ln -s /usr/local/php'.$vn.'/bin/php /usr/local/bin/php');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'ln -s /usr/local/php7/bin/php /usr/bin/php');
+         'ln -s /usr/local/php'.$vn.'/bin/php /usr/bin/php');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'mkdir -vp /usr/local/php7/etc/conf.d','__display__');
-      ($stdout,$stderr)=$handle->cmd($sudo.
-         'cp -v ./php.ini-production /usr/local/php7/etc/php.ini',
+         'cp -v ./php.ini-production /usr/local/php'.$vn.'/etc/php.ini',
          '__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
          "sed -i \'s/post_max_size = 8M/post_max_size = 500M/\' ".
-         "/usr/local/php7/etc/php.ini");
+         "/usr/local/php$vn/etc/php.ini");
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         "sed -i \'s#^.*pdo_mysql.default_socket.*\$#".
+         "pdo_mysql.default_socket = /var/run/mysqld/mysqld.sock#\' ".
+         "/usr/local/php$vn/etc/php.ini");
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         "sed -i \'s#;date.timezone =#date.timezone = \"America/Chicago\"#\' ".
+         "/usr/local/php$vn/etc/php.ini");
       ($stdout,$stderr)=$handle->cmd($sudo.
          "sed -i \'s/upload_max_filesize = 2M/upload_max_filesize = 500M/\' ".
-         "/usr/local/php7/etc/php.ini");
+         "/usr/local/php$vn/etc/php.ini");
       ($stdout,$stderr)=$handle->cmd($sudo.
          "sed -i \'s/max_execution_time = 30/max_execution_time = 7500/\' ".
-         "/usr/local/php7/etc/php.ini");
+         "/usr/local/php$vn/etc/php.ini");
       ($stdout,$stderr)=$handle->cmd($sudo.
          'sed -i \'s/memory_limit = 128M/memory_limit = 256M/\' '.
-         '/usr/local/php7/etc/php.ini');
+         '/usr/local/php'.$vn.'/etc/php.ini');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'mkdir -vp /usr/local/php7/etc/conf.d','__display__');
+         'sed -i \'s/;cgi.fix_pathinfo=1/cgi.fix_pathinfo=0/\' '.
+         '/usr/local/php'.$vn.'/etc/php.ini');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'mkdir -vp /usr/local/php7/etc/php-fpm.d','__display__');
+         'mkdir -vp /usr/local/php'.$vn.'/etc/conf.d','__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'cp -v ./sapi/fpm/www.conf /usr/local/php7/etc/php-fpm.d/www.conf',
+         'mkdir -vp /usr/local/php'.$vn.'/etc/php-fpm.d','__display__');
+      # https://myshell.co.uk/blog/2012/07/adjusting-child-processes-for-php-fpm-nginx/
+      # find DNS server for domain:  dig ns getwisdom.com
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'cp -v ./sapi/fpm/www.conf /usr/local/php'.$vn.
+         '/etc/php-fpm.d/www.conf','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'cp -v ./sapi/fpm/php-fpm.conf /usr/local/php'.$vn.'/etc',
          '__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'cp -v ./sapi/fpm/php-fpm.conf /usr/local/php7/etc/php-fpm.conf',
-         '__display__');
-      my $wcnf=<<END;
-catch_workers_output = yes
-
-php_flag[display_errors] = on
-php_admin_value[error_log] = /usr/local/php7/var/log/fpm-php.www.log
-php_admin_flag[log_errors] = on
-END
-      ($stdout,$stderr)=$handle->cmd(
-         "echo -e \"$wcnf\" | ${sudo}tee -a ".
-         '/usr/local/php7/etc/php-fpm.d/www.conf');
+         'sed -i "s#;pid = run/php-fpm.pid#pid = /var/run/php-fpm.pid#" '.
+         '/usr/local/php'.$vn.'/etc/php-fpm.conf');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'touch /var/log/fpm-php.www.log');
+         'mkdir -vp /var/run/php-fpm','__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'chmod -v 777 /var/log/fpm-php.www.log','__display__');
+         'mkdir -vp /var/log/php-fpm','__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'cp -v ./sapi/fpm/php-fpm.conf /usr/local/php7/etc/php-fpm.conf',
-         '__display__');
+         'sed -i "s#;error_log = log/php-fpm.log#'.
+         'error_log = /var/log/php-fpm/php-fpm.log#" '.
+         '/usr/local/php'.$vn.'/etc/php-fpm.conf');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'sed -i "s#;catch_workers_output = yes#'.
+         'catch_workers_output = yes#" '.
+         '/usr/local/php'.$vn.'/etc/php-fpm.d/www.conf');
       my $zend=<<END;
 ; Zend OPcache
 extension=opcache.so
 END
       ($stdout,$stderr)=$handle->cmd("echo -e \"$zend\" > ".
-         '/usr/local/php7/etc/conf.d/modules.ini');
+         '/usr/local/php'.$vn.'/etc/conf.d/modules.ini');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         "sed -i 's/user = nobody/user = www-data/' ".
-         '/usr/local/php7/etc/php-fpm.d/www.conf');
+         "sed -i 's#listen = 127.0.0.1:9000#".
+         "listen = /var/run/php-fpm/www.sock#' ".
+         '/usr/local/php'.$vn.'/etc/php-fpm.d/www.conf');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         "sed -i 's/group = nobody/group = www-data/' ".
-         '/usr/local/php7/etc/php-fpm.d/www.conf');
+         "sed -i 's/^user = nobody/user = www-data/' ".
+         '/usr/local/php'.$vn.'/etc/php-fpm.d/www.conf');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         "sed -i 's/^group = nobody/group = www-data/' ".
+         '/usr/local/php'.$vn.'/etc/php-fpm.d/www.conf');
       ($stdout,$stderr)=$handle->cmd($sudo.
          "sed -i 's/\;env.PATH./env[PATH]/' ".
-         '/usr/local/php7/etc/php-fpm.d/www.conf');
+         '/usr/local/php'.$vn.'/etc/php-fpm.d/www.conf');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'ln -s /usr/local/php7/sbin/php-fpm /usr/sbin/php-fpm');
-      #
-      # echo-ing/streaming files over ssh can be tricky. Use echo -e
-      #          and replace these characters with thier HEX
-      #          equivalents (use an external editor for quick
-      #          search and replace - and paste back results.
-      #          use copy/paste or cat file and copy/paste results.):
-      #
-      #          !  -   \\x21     `  -  \\x60   * - \\x2A
-      #          "  -   \\x22     \  -  \\x5C
-      #          $  -   \\x24     %  -  \\x25
-      #
-      # https://www.lisenet.com/2014/ - bash approach to conversion
-   my $fpmsrv=<<END;
-[Unit]
-Description=The PHP FastCGI Process Manager
-After=syslog.target network.target
-
-[Service]
-Type=simple
-PIDFile=/run/php-fpm/php-fpm.pid
-ExecStart=/usr/local/php7/sbin/php-fpm --nodaemonize --fpm-config /usr/local/php7/etc/php-fpm.conf
-ExecReload=/bin/kill -USR2 \\x24MAINPID
-
-[Install]
-WantedBy=multi-user.target
-END
-      ($stdout,$stderr)=$handle->cmd("echo -e \"$fpmsrv\" > ".
-         '~/php-fpm.service');
-      ($stdout,$stderr)=$handle->cmd($sudo.'mv -fv ~/php-fpm.service '.
-         '/usr/lib/systemd/system');
-      ($stdout,$stderr)=$handle->cwd("/opt/source");
-      ($stdout,$stderr)=$handle->cmd($sudo.'mkdir -vp /run/php-fpm',
-         '__display__');
+         "sed -i 's/;listen.mode = 0660/listen.mode = 0666/' ".
+         '/usr/local/php'.$vn.'/etc/php-fpm.d/www.conf');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         'chkconfig --levels 235 php-fpm on');
-      ($stdout,$stderr)=$handle->cmd($sudo.'service php-fpm start',
-         '__display__');
+         'ln -s /usr/local/php'.$vn.'/sbin/php-fpm /usr/sbin/php-fpm');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'cp -v /opt/source/php-src/sapi/fpm/php-fpm.service '.
+         '/etc/systemd/system','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'sed -i "s#PIDFile=/usr/local/php'.$vn.'#PIDFile=#" '.
+         '/etc/systemd/system/php-fpm.service');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'systemctl daemon-reload');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'systemctl enable php-fpm.service','__display__');
+      sleep 2;
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'service php-fpm start','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'service php-fpm status -l','__display__');
       $prompt=$handle->prompt();
       ($stdout,$stderr)=$handle->cwd('/opt/source');
       ($stdout,$stderr)=$handle->cmd($sudo.
@@ -496,26 +752,70 @@ END
          }
       }
       ($stdout,$stderr)=$handle->cmd($sudo.
-         '/usr/local/php7/bin/pecl channel-update pecl.php.net',
+         'wget --random-wait --progress=dot '.
+         'http://curl.haxx.se/ca/cacert.pem '.
+         '--output-document /usr/local/ssl/cert.pem',
+         '__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         '/usr/local/php'.$vn.'/bin/pecl channel-update pecl.php.net',
          '__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
          'wget -qO- https://pecl.php.net/package/mailparse');
       $stdout=~s/^.*?get\/(mailparse-.*?).tgz.*$/$1/s;
       my $version=$stdout;
       ($stdout,$stderr)=$handle->cmd($sudo.
-         "/usr/local/php7/bin/pecl install $version",
+         "/usr/local/php$vn/bin/pecl install $version",
          '__display__');
+      $version='';
       ($stdout,$stderr)=$handle->cmd($sudo.
          'bash -c "echo extension=mailparse.so > '.
-         '/usr/local/php7/etc/conf.d/mailparse.ini"');
+         '/usr/local/php'.$vn.'/etc/conf.d/mailparse.ini"');
+      ($stdout,$stderr)=$handle->cwd('/opt/source');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'wget --random-wait --progress=dot '.
+         'https://download.imagemagick.org/'.
+         'ImageMagick/download/ImageMagick.zip',
+         '__display__');
+      sleep 2;
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'unzip -o ImageMagick.zip','__display__');
+      ($stdout,$stderr)=$handle->cwd('ImageMag*');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         './configure --with-modules','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'make install','3600','__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'ldconfig -v /usr/local/lib','__display__');
+      ($stdout,$stderr)=$handle->cwd('/opt/source');
+      ($stdout,$stderr)=$handle->cmd($sudo.
+         'wget -qO- https://pecl.php.net/package/imagick','300');
+      $stdout=~s/^.*?get\/(imagick-.*?).tgz.*$/$1/s;
+      $version=$stdout;
+      $handle->print($sudo.
+         "/usr/local/php$vn/bin/pecl install $version");
+      $prompt=$handle->prompt();
+      while (1) {
+         my $output.=fetch($handle);
+         last if $output=~/$prompt/;
+         print $output;
+         if (-1<index $output,'autodetect') {
+            $handle->print('');
+            $output='';
+         } sleep 1;
+      }
+      sleep 2;
       ($stdout,$stderr)=$handle->cmd($sudo.'service php-fpm start',
          '__display__');
+      ($stdout,$stderr)=$handle->cmd($sudo.'service php-fpm status -l',
+         '__display__');
    } elsif (-e '/opt/cpanel/ea-php70') {
       ($stdout,$stderr)=$handle->cmd($sudo.
          'cp -v /opt/cpanel/ea-php70/root/etc/php-fpm.d/www.conf.default '.
          '/opt/cpanel/ea-php70/root/etc/php-fpm.d/www.conf','__display__');
       ($stdout,$stderr)=$handle->cmd($sudo.
-         "/etc/init.d/ea-php70-php-fpm start",'__display__');
+         '/etc/init.d/ea-php70-php-fpm start','__display__');
+   } else {
+      print "php is up to date.\n";
    }
 
    my $thanks=<<'END';
